/*

   file: ListDictionary.cc

   Content: ListDictionary source code.

   Revision History:

   Creation - May 12, 2003 by M. Jansen.
*/


#include <iostream.h>
#include "ListDictionary.h"

template <class Key, class Item>
ListDictionary<Key, Item>::ListDictionary() : Dictionary<Key, Item>(true)
/*
   purpose : default construct a ListDictionary object
*/
{
   head = NULL;
}

template <class Key, class Item>
ListDictionary<Key, Item>::ListDictionary(bool v) : Dictionary<Key, Item>(v)
/*
   purpose : construct a ListDictionary Object.

   input   : v = true <=> Keys must be unique.
*/
{
   head = NULL;
} 

template <class Key, class Item>
ListDictionary<Key, Item>::~ListDictionary()
/*
   purpose : destroy a ListDictionary Object.
*/
{
   ListItem* pi;

   while (head != 0) //free up entire list
   {
      pi = head->next;
      delete head; // destruction of the ListItem (includes  
      head = pi;   // destruction of DictionaryItem data).
   }
}

template <class Key, class Item>
bool ListDictionary<Key, Item>::insert(Key& key, Item& item)
/*
         purpose: add an Key, Item pair to the Dictionary.

         input  : key  - key that is to be inserted,
                  item - item to be associated with object key.

         result : A copy is made of the objects key and item,
                  and the copied pair is stored.

         returns: true <=> succesful.

         If multiple keys are not allowed, this function will
         return false, in case there is already a Key object kk,
         such that kk == key. (Key objects must have operator== defined.)
         In this case no change is made to the Dictionary.

         If multiple Keys are allowed, this method will always return
         true and add the pair (key, item) to the dictionary.
*/
{
   ListItem* pli;
   Key* pkey;
   Item* pitem;

   if (hasUniqueKeys())
   {
      pitem = find(key);
      if (pitem != NULL)
         return false;
   }

   pli = new ListItem();
   pkey = new Key(key); // make copy
   pitem = new Item(item); // make copy

   pli->item = pitem;
   pli->key = pkey;
   pli->next = head;
   head = pli;
   return true; 
}

template <class Key, class Item>
Item* ListDictionary<Key, Item>::find(Key& key)
/*
   purpose: retrieve an item from the Dictionary.

   input  : key is the Key object to perform the look-up with.
   returns: a ptr to an item in the dictionary with Key object == key,
            or NULL if not found.

   Note   : As of yet, there is no means for retrieving
            all items, in case non-unique Keys are allowed.
*/
{
   ListItem* current = head;
   char notfound = 1;

   while (current !=0 && notfound)
   {
     if (key == *(current->key))
        notfound = 0;
     else
        current = current->next;
   }
   if (notfound)
      return NULL;
   else
      return current->item;
}

template <class Key, class Item>
void ListDictionary<Key, Item>::remove(Key& key)
/*
   purpose: remove an item from the dictionary.

   input  : key is the Key object to remove with.
   result : The item with Key == key (if present)
            is removed from the Dictionary. In case multiple Keys
            are allowed, only one such item is removed.
*/
{
   ListItem* current = head;
   ListItem* prev = NULL;
   bool found = false;

   while (current !=NULL & !found)
   {
     if (key == *(current->key))
        found = true;
     else
     {
        prev = current;
        current = current->next;
     }
   }

   if (found)
   {
      if (prev == NULL)
      {
         head = current->next;
         delete current; 
      }
      else
      {
         prev->next = current->next;
         delete current;
      }
   }
}

template <class Key, class Item>
void ListDictionary<Key, Item>::removeAll(Key& key)
/*
   purpose: remove multiple items from the dictionary.

   input  : key is the Key object to remove with.
   result : All Items with Key == key are removed
            from the Dictionary.
*/
{
   while (find(key))
      remove(key);
}

